#define BUILDING_NODE_EXTENSION
#include <node.h>
#include "Arduino.h"
#include "common.h"

using namespace v8;

#ifdef __NODE_V0_11__
static void analogWrite(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = Isolate::GetCurrent();
  HandleScope scope(isolate);
#else
Handle<Value> analogWrite(const Arguments& args) {
  HandleScope scope;
#endif

  if (args.Length() < 2) {
#ifdef __NODE_V0_11__
      isolate->ThrowException(Exception::TypeError(v8::String::NewFromUtf8(isolate, "Wrong number of arguments")));
      return;
#else
      ThrowException(Exception::TypeError(v8::String::New("Wrong number of arguments")));
      return scope.Close(Undefined());
#endif
    }

    if (!args[0]->IsNumber() || !args[1]->IsNumber()) {
#ifdef __NODE_V0_11__
      isolate->ThrowException(Exception::TypeError(v8::String::NewFromUtf8(isolate, "Wrong arguments")));
      return;
#else
      ThrowException(Exception::TypeError(v8::String::New("Wrong arguments")));
      return scope.Close(Undefined());
#endif
    }

    int pin = args[0]->NumberValue();
    int value = args[1]->NumberValue();

    analogWrite(pin, value);

#ifdef __NODE_V0_11__
  return;
#else
  return scope.Close(Undefined());
#endif
}

#ifdef __NODE_V0_11__
static void analogRead(const FunctionCallbackInfo<Value>& args) {
  Isolate* isolate = Isolate::GetCurrent();
  HandleScope scope(isolate);
#else
Handle<Value> analogRead(const Arguments& args) {
  HandleScope scope;
#endif

  if (args.Length() < 1) {
#ifdef __NODE_V0_11__
      isolate->ThrowException(Exception::TypeError(v8::String::NewFromUtf8(isolate, "Wrong number of arguments")));
      return;
#else
      ThrowException(Exception::TypeError(v8::String::New("Wrong number of arguments")));
      return scope.Close(Undefined());
#endif
    }

    if (!args[0]->IsNumber()) {
#ifdef __NODE_V0_11__
      isolate->ThrowException(Exception::TypeError(v8::String::NewFromUtf8(isolate, "Wrong number of arguments")));
      return;
#else
      ThrowException(Exception::TypeError(v8::String::New("Wrong arguments")));
      return scope.Close(Undefined());
#endif
    }

    int pin = args[0]->NumberValue();

   int value = analogRead(pin);

#ifdef __NODE_V0_11__
   Local<Number> num = Number::New(isolate, value);
   args.GetReturnValue().Set(num);
#else
   Local<Number> num = Number::New(value);
   return scope.Close(num);
#endif
}

void Init(Handle<Object> exports)
{
#ifdef __NODE_V0_11__
  Isolate* isolate = Isolate::GetCurrent();
#endif
	init();

#ifdef __NODE_V0_11__
  exports->Set(v8::String::NewFromUtf8(isolate, "analogWrite"),
      FunctionTemplate::New(isolate, analogWrite)->GetFunction());
#else
  exports->Set(v8::String::NewSymbol("analogWrite"),
      FunctionTemplate::New(analogWrite)->GetFunction());
#endif

#ifdef __NODE_V0_11__
  exports->Set(v8::String::NewFromUtf8(isolate, "analogRead"),
            FunctionTemplate::New(isolate, analogRead)->GetFunction());
#else
	exports->Set(v8::String::NewSymbol("analogRead"),
            FunctionTemplate::New(analogRead)->GetFunction());
#endif

}

NODE_MODULE(node_pcduino_analog, Init)
